@@ -1,44 +1,96 @@
-# IP subnet calculator
-# (C) 2007 Wijnand 'tehmaze' Modderman - http://tehmaze.com
-# BSD License
-#
-# ABOUT
-#  This module allows you to perform network calculations.
-#
-# CHANGELOG
-#  2009-03-23: Added IPv4 short-hand form support, thanks to VeXocide.
-#  2007-10-26: Added IPv6 support, as well as a lot of other functions, 
-#              refactored the calculations.
-#  2007-10-25: Initial writeup, because I could not find any other workable
-#              implementation.
-#
-# TODO
-#  * add CLI parser
-#
-# REFERENCES
-#  * http://www.estoile.com/links/ipv6.pdf
-#  * http://www.iana.org/assignments/ipv4-address-space
-#  * http://www.iana.org/assignments/multicast-addresses
-#  * http://www.iana.org/assignments/ipv6-address-space
-#  * http://www.iana.org/assignments/ipv6-tla-assignments
-#  * http://www.iana.org/assignments/ipv6-multicast-addresses
-#  * http://www.iana.org/assignments/ipv6-anycast-addresses
-#
-# THANKS (testing, tips)
-#  * Bastiaan (trbs)
-#  * Peter van Dijk (Habbie)
-#  * Hans van Kranenburg (Knorrie)
-#  * Jeroen Habraken (VeXocide)
-#
+# -*- coding: utf-8 -*-
 
-__version__ = '0.3'
+'''
+====================================
+ :mod:`ipcalc` IP subnet calculator
+====================================
+.. moduleauthor:: Wijnand Modderman <http://tehmaze.com/>
+.. note:: BSD License
+
+About
+=====
+
+This module allows you to perform network calculations.
+
+Changelog
+=========
+
+    ========== ===============================================================
+    Date       Changes
+    ========== ===============================================================
+    2009-04-11 Added docstrings.
+    2009-03-23 Added IPv4 short-hand form support, thanks to VeXocide.
+    2007-10-26 Added IPv6 support, as well as a lot of other functions, 
+               refactored the calculations.
+    2007-10-25 Initial writeup, because I could not find any other workable
+               implementation.
+    ========== ===============================================================
+
+Todo
+====
+
+Todo:
+ * add CLI parser
+
+References
+==========
+
+References:
+ * http://www.estoile.com/links/ipv6.pdf
+ * http://www.iana.org/assignments/ipv4-address-space
+ * http://www.iana.org/assignments/multicast-addresses
+ * http://www.iana.org/assignments/ipv6-address-space
+ * http://www.iana.org/assignments/ipv6-tla-assignments
+ * http://www.iana.org/assignments/ipv6-multicast-addresses
+ * http://www.iana.org/assignments/ipv6-anycast-addresses
+
+Thanks
+======
+
+I wish to thank the following people for their input:
+ * Bastiaan (trbs)
+ * Peter van Dijk (Habbie)
+ * Hans van Kranenburg (Knorrie)
+ * Jeroen Habraken (VeXocide)
+ * Torbjörn Lönnemark
+
+'''
+
+__version__ = '0.5.2'
+
+import socket
+
+
+try:
+    bin(42)
+except NameError:
+    def bin(x):
+        '''
+        Stringifies an int or long in base 2.
+        '''
+        if x < 0: 
+            return '-' + bin(-x)
+        out = []
+        if x == 0: 
+            out.append('0')
+        while x > 0:
+            out.append('01'[x & 1])
+            x >>= 1
+            pass
+        try: 
+            return '0b' + ''.join(reversed(out))
+        except NameError, ne2: 
+            out.reverse()
+        return '0b' + ''.join(out) 
 
-import types, socket
 
 class IP(object):
     '''
     Represents a single IP address.
 
+    :param ip: the ip address
+    :type ip: :class:`IP` or str or long or int 
+
     >>> localhost = IP("127.0.0.1")
     >>> print localhost
     127.0.0.1
@@ -109,12 +161,14 @@ class IP(object):
         self.mask = mask
         self.v = 0
         # Parse input
-        if isinstance(ip, IP):
+        if ip is None:
+            raise ValueError, "Can not pass None"
+        elif isinstance(ip, IP):
             self.ip = ip.ip
             self.dq = ip.dq
             self.v = ip.v
             self.mask = ip.mask
-        elif type(ip) in [types.IntType, types.LongType]:
+        elif isinstance(ip, (int, long)):
             self.ip = long(ip)
             if self.ip <= 0xffffffff:
                 self.v = version or 4
@@ -123,10 +177,10 @@ class IP(object):
                 self.v = version or 4
                 self.dq = self._itodq(ip)
         else:
-            # If string is in CIDR notation
+            # If string is in CIDR or netmask notation
             if '/' in ip:
                 ip, mask = ip.split('/', 1)
-                self.mask = int(mask)
+                self.mask = mask
             self.v = version or 0
             self.dq = ip
             self.ip = self._dqtoi(ip)
@@ -134,13 +188,26 @@ class IP(object):
         # Netmask defaults to one ip
         if self.mask is None:
             self.mask = self.v == 4 and 32 or 128
+        # Netmask is numeric CIDR subnet
+        elif isinstance(self.mask, (int, long)) or self.mask.isdigit():
+            self.mask = int(self.mask)
+        # Netmask is in subnet notation
+        elif isinstance(self.mask, basestring):
+            limit = [32, 128][':' in self.mask]
+            inverted = ~self._dqtoi(self.mask)
+            count = 0
+            while inverted & 2**count:
+                count += 1
+            self.mask = (limit - count)
+        else:
+            raise ValueError, "Invalid netmask"
         # Validate subnet size
         if self.v == 6:
             self.dq = self._itodq(self.ip)
-            if self.mask < 0 or self.mask > 128:
+            if not 0 <= self.mask <= 128:
                 raise ValueError, "IPv6 subnet size must be between 0 and 128"
         elif self.v == 4:
-            if self.mask < 0 or self.mask > 32:
+            if not 0 <= self.mask <= 32:
                 raise ValueError, "IPv4 subnet size must be between 0 and 32"
 
     def bin(self):
@@ -151,10 +218,7 @@ class IP(object):
         >>> print ip.bin()
         01111111000000000000000000000001
         '''
-        h = hex(self.ip).lower().rstrip('l')
-        b = ''.join(self._bitmask[x] for x in h[2:])
-        l = self.v == 4 and 32 or 128
-        return ''.join('0' for x in xrange(len(b), l)) + b
+        return bin(self.ip).split('b')[1].rjust(self.mask, '0')
 
     def hex(self):
         '''
@@ -181,7 +245,7 @@ class IP(object):
         4
         '''
         return self.v
-   
+
     def info(self):
         '''
         Show IANA allocation information for the current IP address.
@@ -196,7 +260,7 @@ class IP(object):
             if self._range[self.v].has_key(b[:i]):
                 return self._range[self.v][b[:i]]
         return 'UNKNOWN'
- 
+
     def _dqtoi(self, dq):
         '''
         Convert dotquad or hextet to long.
@@ -241,7 +305,7 @@ class IP(object):
             for h in hx:
                 if len(h) < 4:
                     h = '%04x' % int(h, 16)
-                if 0 > int(h, 16) > 0xffff:
+                if not 0 <= int(h, 16) <= 0xffff:
                     raise ValueError, "%r: IPv6 address invalid: hextets should be between 0x0000 and 0xffff" % dq
                 ip += h
             self.v = 6
@@ -250,7 +314,7 @@ class IP(object):
             # Assume full heximal notation
             self.v = 6
             return long(h, 16)
-        
+
         # IPv4
         if '.' in dq:
             q = dq.split('.')
@@ -258,15 +322,15 @@ class IP(object):
             if len(q) > 4:
                 raise ValueError, "%r: IPv4 address invalid: more than 4 bytes" % dq
             for x in q:
-                if 0 > int(x) > 255:
+                if not 0 <= int(x) <= 255:
                     raise ValueError, "%r: IPv4 address invalid: bytes should be between 0 and 255" % dq
             while len(q) < 4:
                 q.insert(1, '0')
             self.v = 4
             return sum(long(byte) << 8 * index for index, byte in enumerate(q))
-    
+
         raise ValueError, "Invalid address input"
-       
+
     def _itodq(self, n):
         '''
         Convert long to dotquad or hextet.
@@ -293,6 +357,21 @@ class IP(object):
     def __long__(self):
         return self.ip
 
+    def __lt__(self, other):
+        return long(self) < long(IP(other))
+
+    def __le__(self, other):
+        return long(self) <= long(IP(other))
+
+    def __ge__(self, other):
+        return long(self) >= long(IP(other))
+
+    def __gt__(self, other):
+        return long(self) > long(IP(other))
+
+    def __eq__(self, other):
+        return long(self) == long(IP(other))
+
     def size(self):
         return 1
 
@@ -301,8 +380,8 @@ class IP(object):
         Return a new <IP> object with a copy of this one.
 
         >>> ip = IP('127.0.0.1')
-        >>> ip.clone()
-        <ipcalc.IP object at 0xb7d4d18c>
+        >>> ip.clone() # doctest: +ELLIPSIS
+        <ipcalc.IP object at 0x...>
         '''
         return IP(self)
 
@@ -325,6 +404,25 @@ class IP(object):
             else:
                 return ValueError, "%r: IPv6 address is not IPv4 compatible, nor a 6-to-4 IP" % self.dq
 
+    @classmethod
+    def from_bin(cls, value):
+        value = value.lstrip('b')
+        if len(value) == 32:
+            return cls(int(value, 2))
+        elif len(value) == 128:
+            return cls(long(value, 2))
+        else:
+            return ValueError, "%r: invalid binary notation" % (value,)
+
+    @classmethod
+    def from_hex(cls, value):
+        if len(value) == 8:
+            return cls(int(value, 16))
+        elif len(value) == 32:
+            return cls(long(value, 16))
+        else:
+            raise ValueError, "%r: invalid hexadecimal notation" % (value,)
+
     def to_ipv6(self, type='6-to-4'):
         '''
         Convert (an IPv4) IP address to an IPv6 address.
@@ -347,11 +445,18 @@ class IP(object):
         Used for comparisons.
         '''
         return (self.dq, self.mask)
-    
+
+
 class Network(IP):
     '''
     Network slice calculations.
 
+    :param ip: network address
+    :type ip: :class:`IP` or str or long or int 
+    :param mask: netmask
+    :type mask: int or str
+
+
     >>> localnet = Network('127.0.0.1/8')
     >>> print localnet
     127.0.0.1
@@ -379,7 +484,7 @@ class Network(IP):
         127.0.0.0
         '''
         return IP(self.ip & long(self.netmask()), version=self.version())
-    
+
     def broadcast(self):
         '''
         Broadcast address.
@@ -441,6 +546,9 @@ class Network(IP):
     def __ge__(self, other):
         return self.size() >= IP(other).size()
 
+    def __eq__(self, other):
+        return self.size() == IP(other).size()
+
     def __iter__(self):
         '''
         Generate a range of ip addresses within the network.
@@ -453,13 +561,16 @@ class Network(IP):
         192.168.114.2
         192.168.114.3
         '''
-        for ip in [IP(long(self)+x) for x in xrange(0, self.size())]:
+        for ip in (IP(long(self)+x) for x in xrange(0, self.size())):
             yield ip
 
     def has_key(self, ip):
         '''
         Check if the given ip is part of the network.
 
+        :param ip: the ip address
+        :type ip: :class:`IP` or str or long or int 
+
         >>> net = Network('192.0.2.0/24')
         >>> net.has_key('192.168.2.0')
         False
@@ -478,12 +589,16 @@ class Network(IP):
         '''
         return 2 ** ((self.version() == 4 and 32 or 128) - self.mask)
 
+
 if __name__ == '__main__':
     tests = [
         ('192.168.114.42', 23, ['192.168.0.1', '192.168.114.128', '10.0.0.1']),
         ('123::', 128, ['123:456::', '::1', '123::456']),
         ('::42', 64, ['::1', '1::']),
-        ('2001:dead:beef:1:c01d:c01a::', 48, ['2001:dead:beef:babe::'])
+        ('2001:dead:beef:1:c01d:c01a::', 48, ['2001:dead:beef:babe::']),
+        ('10.10.0.0', '255.255.255.0', ['10.10.0.20', '10.10.10.20']),
+        ('2001:dead:beef:1:c01d:c01a::', 'ffff:ffff:ffff::', ['2001:dead:beef:babe::']),
+        ('10.10.0.0/255.255.240.0', None, ['10.10.0.20', '10.10.250.0']),
         ]
 
     for ip, mask, test_ip in tests:
@@ -506,4 +621,3 @@ if __name__ == '__main__':
         print 'last host.:', net.host_last()
         for ip in test_ip:
             print '%s in network: ' % ip, ip in net
-
